%% Improcess.m claculates the area and length of wound/cut in fish after surgical
%  tag implantation.
%  Fueled by IPA, Heavy Metal and the vodoo power of Matlab !!!
%  Requires Matlab Image Processing Toolbox
%  6/3/2013
%  Bishes Rayamajhi


clear;%clc;
close all;
option = 2; % 1 - gray scale, 2 - red pixels extraction
files_dir = dir('d:\IAMT\2013-06-30\right\*.jpg');
results_mat = cell(length(files_dir),3);

outPath = '.\processed_images\';
if(~exist(outPath,'dir'))
    mkdir(outPath);
end 



for i = 1:5%length(files_dir)
    filename = files_dir(i).name;
    disp(filename);
    I = imread(filename);
    IG = rgb2gray(I);
    image_info = imfinfo(filename);
    
    [pixel_dist, pixel_area] = getscale2(I,image_info,filename);
    if pixel_dist == inf;
        results_mat(i,1) = {filename};
        results_mat(i,2) = {'n/a'};
        results_mat(i,3) = {'n/a'};
        continue
    end
    %% image processing section

    thresh1 = graythresh(IG);
    cord = [image_info.Width/6 image_info.Height/4 image_info.Width*3/4 image_info.Height/2];
    IGcr = imcrop(I,cord);
    imcr = imcrop(IGcr);
    
    %% 
       
    if option == 1 
    opt = ' using Gray scale extraction';             % Gray scale image extraction
    imcr_red  = imadjust(imcr,[.6 .4 .4; .9 .6 .6],[]);  % For wounds with dark shadows around
    imcr = rgb2gray(imcr);
    imcr_filt = medfilt2(imcr,[9 9]);    
    imcr_filt = imadjust(imcr_filt);                                
                                     
  
    elseif option == 2
    opt = ' using Red Extraction';                     % Red pixel intensity extraction                                                      
    imcr_red  = imadjust(imcr,[.6 .4 .4; .9 .6 .6],[]);   % For normal pictures  (works good for ...
    imcr_gr = rgb2gray(imcr_red);                         % blurred wound pictures).
    imcr_filt = medfilt2(imcr_gr,[9 9]);
    end
    
%%

%     thresh2 = graythresh(imcr)
%     ave = (thresh1 + thresh2)/2;

%     imcr = imadjust(imcr); % using imadjust fluctuates the output as crop area
                            % change varies its default contrast threshold                      

    imbw = im2bw(imcr_filt,thresh1-.025);
    imbw1 = ~imbw;
    imbw1 = imclearborder(imbw1);
    imbw1 = bwareaopen(imbw1,300);

    stats = regionprops(imbw1,'Area');
     
    if numel(stats) ~= 1
    formatSpec = 'Error in finding area in %s, needs to be manually checked!\n';
    fprintf(formatSpec,filename);
    results_mat(i,1) = {filename};
    results_mat(i,2) = {'n/a'};
    results_mat(i,3) = {'n/a'};
    continue
    end
    
    
    area  = stats.Area;
    area = area/pixel_area;
    formatSpec = 'Wound area = %0.3f sq cm\n';
    fprintf(formatSpec,area);
    area = sprintf('%0.3f',area);

%% Plotting wound boundary and length 

    [B,L] = bwboundaries(imbw1,'noholes');
    subplot(2,2,[2 4])
    set(gca,'Position', [0.5702 0.2469 0.3113 0.5506]);
    imshow(label2rgb(L, @jet, [.5 .5 .5]))
    title(strcat('Tracked Wound Plot',opt),'FontWeight','bold');
    hold on

    boundary = cell2mat(B); 
    plot(boundary(:,2), boundary(:,1), 'r', 'LineWidth', 2)

    wound_dist_max = getlength(B);
    max_length = wound_dist_max(1)/pixel_dist;
    formatSpec = 'Wound length = %0.3f cm\n';
    fprintf(formatSpec,max_length);
    max_length = sprintf('%0.3f',max_length);
    plot([wound_dist_max(2) wound_dist_max(4)],[wound_dist_max(3) wound_dist_max(5)]...
        ,'b','LineWidth',1.5);
    len_str = strcat(num2str(max_length),' cm');
    area_str = strcat(num2str(area),' sq cm');
    annotation('textbox',[0.6452 0.1027 0.1615 0.0889],'String',{'Area = ',...
       area_str,'Length =',len_str},'FitBoxToText','on');
    hold off
    
    subplot(2,2,1)
    set(gca,'Position',[0.182 0.542 0.309 0.296]);
    imshow(imcr)
    title('Actual Wound Image','FontWeight','bold');
    
    subplot(2,2,3)
    set(gca,'Position', [0.182 0.157 0.309 0.298]);
    imshow(imcr_red)
    title('Wound Image in Red Extract ','FontWeight','bold');
    
    filename_disp = strrep(filename,'_','-');
    annotation('textbox',[0.25 0.96 0.48 0.03],...
        'String',filename_disp, 'Fontsize',14,'Linestyle','none');
    pause(3)
    set(gcf,'paperpositionmode','auto');
    print('-dpng','-r300',strcat(outPath, 'Processed_Image_',filename_disp));
    clf('reset');
    
    
    if str2double(area) < 0.015 || str2double(area) > 0.040
    results_mat(i,1) = {filename};
    results_mat(i,2) = {'n/a'};
    results_mat(i,3) = {'n/a'};
    disp('Area out of range, recheck manually!');
    else
    results_mat(i,1) = {filename};
    results_mat(i,2) = {area};
    results_mat(i,3) = {max_length};
    end
end

close all; 
columnTitle = {'Filename','Area(cm^2)','Length(cm)'};
xlswrite('results.xls',columnTitle,'sheet1');
xlswrite('results.xls',results_mat,'sheet1','A2');
clear all;
